#include "display.h"

void
Display::draw()
	{
	if ( piece != NULL )
		{
		piece->draw();
		}
	for ( int i=0; i<BOARD_WIDTH; i++ )
		{
		for ( int j=0; j<BOARD_HEIGHT; j++ )
			{
			if ( boardJewels[i][j] != NULL )
				{
				// printf( "drawing i: %d j: %d\n", i, j );
				boardJewels[i][j]->draw();
				}
			}
		}	

	// draw scoreboard
	scoreBoard->draw();
	}

void 
Display::update()
	{
	if ( state == FALLING_PIECE )
		{
		float posy = piece->getYPos();
		int destx = piece->getXDest();
		//int desty = piece->getYDest();
		int newdest = getTopOfColumn( destx );
		//if ( posy > (float) newdest - PIECE_DONE_THRESH 
		//		&& posy < (float) newdest + PIECE_DONE_THRESH )
		if ( posy <= (float) newdest + PIECE_DONE_THRESH)
			{
			// printf( "piece done\n" );
			scoreBoard->addScore( PLACEPIECESCORE );

			for( int i=0; i<3; i++ )
				{
				Jewel *newjewel = piece->getJewel( i );
				newjewel->quantitizePos();
				int xpos = newjewel->getXDest();
				int ypos = (int) newjewel->getYPos();
				if ( ypos > BOARD_HEIGHT - 1 )
					{
					if ( livesLeft < 1 )
						{
						state = GAME_OVER_MAN;
						return;
						}
					else
						{
						livesLeft--;
						state = CLEAR_BOARD;
						return;
						}
					}	
				newjewel->setYDest( ypos );
				
				//int ydest = newjewel->getYDest();
				//printf( "x: %d y: %d ydest: %d style: %d\n", 
				//	xpos, ypos, ydest, newjewel->getStyle()  );	
				
				board[ xpos ][ ypos ] = newjewel->getStyle();
				boardJewels[ xpos ][ ypos ] = newjewel;
				piece->stopDrawingJewels();
				}
			checkForMatches();	
			}
		else
			{
			// set for new dest on this column
			//int mody = newdest - desty;
			piece->update();
			}
		}
	else if ( state == GRAVITY_WORK )
		{
		// printf( "gravity\n" );

		// gotta award more points for this
		explodeLevel++;

		if ( !doneReDest )
			{
			doneReDest = 1;
			for ( int i=0; i<BOARD_WIDTH; i++ )
				{
				int count = 0;
				for ( int j=0; j<BOARD_HEIGHT; j++ )
					{
					if ( board[i][j] != -1 )
						{
						int destx = boardJewels[i][j]->getYDest();
						if ( count != destx )
							{
							// printf( "[%d][%d] setting ydest to %d\n", i, j, count );
							boardJewels[i][j]->setYDest( count );
							boardJewels[i][j]->setFallRate( GRAV_FALL_RATE );
							board[i][count] = boardJewels[i][j]->getStyle();
							board[i][j] = -1;
							boardJewels[i][count] = boardJewels[i][j];
							boardJewels[i][j] = NULL;
							}
						count++;	
						}
					}
				}
			}
		else
			{
			// waiting for jewels to fall all the way
			int boardActive = updateBoard();
			// printf( "timer1: %f\n", timer );
			if ( !boardActive )
				{
				//printf( "board not active\n" );
				doneReDest = 0;
				checkForMatches();
				}
			}
		}
	else if ( state == EXPLODING )
		{
		//printf( "exploding\n" );
		// primitive, but it'll have to do for now
		for ( int i=0; i<BOARD_WIDTH; i++ )
			{
			for ( int j=0; j<BOARD_HEIGHT; j++ )
				{
				if ( explode[i][j] == 1 )
					{
					delete boardJewels[i][j];
					boardJewels[i][j] = NULL;
					//printf( "deleting [%d][%d] to -1\n", i, j );
					board[i][j] = -1;
					explode[i][j] = 0;
					scoreBoard->addScore( 
						BASESCOREPERPIECE +
						(explodeLevel * SCOREPERPIECEMULT) );
					}
				}	
			}
		state = GRAVITY_WORK;	
		}
	else if ( state == CLEAR_BOARD )
		{
		clearBoard();
		state = FALLING_PIECE;
		newPiece();
		}
	else if ( state == GAME_OVER_MAN )
		{
		printf( "GAME OVER MAN!\n" );
		exit(0);
		}
	}

void
Display::left()
	{
	if ( piece->getDoDraw() )
		{
		int destx = piece->getXDest();
		int desty = (int) piece->getYPos();
		if ( destx > 0 && board[destx-1][desty] == -1 )
			{
			piece->updateXDest( -1 );
			}
		}	
	}
	
void
Display::right()
	{
	if ( piece->getDoDraw() )
		{
		int destx = piece->getXDest();
		int desty = (int) piece->getYPos();
		if ( destx < BOARD_WIDTH - 1 && board[destx+1][desty] == -1 )
			{
			piece->updateXDest( 1 );
			}
		}
	}
	
void
Display::fastDown()
	{
	if ( piece->getDoDraw() )
		{
		int destx = piece->getXDest();
		int newdest = getTopOfColumn( destx );
		piece->setYPos( newdest );
		}
	}

void
Display::down()
	{
	if ( piece->getDoDraw() )
		{
		piece->downFast();
		}
	}

void
Display::flip()
	{
	if ( piece->getDoDraw() )
		{
		piece->flip();
		}
	}

int
Display::getTopOfColumn( int colnum )
	{
	for ( int i = BOARD_HEIGHT - 1; i >= 0; i-- )
		{
		//printf( "[%d][%d] board: %d\n", colnum, i, board[ colnum ][i] );
		if ( board[ colnum ][i] != -1 )
			{
			return i + 1;
			}
		}
	return 0;
	}

void
Display::checkForMatches()
	{
	int match = 0;

	/*
	printf( "\n" );
	for ( int j=BOARD_HEIGHT-1; j>=0; j-- )
		{
		for ( int i=0; i<BOARD_WIDTH; i++ )
			{
			printf( "%d  ", board[i][j] );
			}
		printf("\n");
		}
	*/	
	
	// Check each jewel
	for ( int i=0; i<BOARD_WIDTH; i++ )
		{
		for ( int j=0; j<BOARD_HEIGHT; j++ )
			{
			if ( board[i][j] == -1 )
				{
				// no jewel here
				continue;
				}
			// got jewel, check surrounding jewels	
			int curJewelStyle = boardJewels[i][j]->getStyle();
			for ( int k=0; k<2; k++ )
				{
				for ( int m=-1; m<2; m++ )
					{
					if ( (k == 0 && m == -1) || (k == 0 && m == 0) )
						{
						// no need to check these
						continue;
						}
					if ( i + k < 0 || i + k > BOARD_WIDTH - 1
							|| j + m < 0 || j + m > BOARD_HEIGHT - 1 
							|| board[ i+k ][ j+m ] == -1 )
						{
						// no surrounding jewel here
						continue;
						}
					int surJewelStyle = board[ i+k ][ j+m ];
					if ( curJewelStyle == surJewelStyle )
						{
						// We got 2 in a row, lets try for a third
						if ( board[ i+(2*k) ][ j+(2*m) ] == surJewelStyle )
							{
							// We got a match!  Mark em to explode!
							explode[i][j] = 1;
							explode[ i+k ][ j+m ] = 1;
							explode[ i+(2*k) ][ j+(2*m) ] = 1;
							//printf( "exploding[%d][%d]\n", i, j );
							//printf( "exploding[%d][%d]\n", i+k, j+m );
							//printf( "exploding[%d][%d]\n", i+(2*k), j+(2*m) );
							match = 1;
							}
						}	
					}
				}	
			}
		}	

	if ( match )
		{
		state = EXPLODING;
		}
	else
		{
		newPiece();
		state = FALLING_PIECE;
		}
	}

void
Display::clearBoard()
	{
	for ( int i=0; i<BOARD_WIDTH; i++ )
		{
		for ( int j=0; j<BOARD_HEIGHT; j++ )
			{
			board[i][j] = -1;
			if ( boardJewels[i][j] != NULL )
				{
				delete boardJewels[i][j];
				boardJewels[i][j] = NULL;
				}
			explode[i][j] = 0;	
			}
		}
	}	

int
Display::updateBoard()
	{
	int active = 0;
	for ( int i=0; i<BOARD_WIDTH; i++ )
		{
		for ( int j=0; j<BOARD_HEIGHT; j++ )
			{
			if ( board[i][j] != -1 )
				{
				if ( boardJewels[i][j]->isMoving() )
					{
					active = 1;
					}
				boardJewels[i][j]->update();
				// float posy = boardJewels[i][j]->getYPos();
				// int desty = boardJewels[i][j]->getYDest();
				// printf( "jewel[%d][%d] posy: %f desty %d\n", i, j, posy, desty );
				}
			}
		}
	return active;	
	}	

