#ifndef DISPLAY_H
#define DISPLAY_H

#include "base.h"
#include "jewel.h"
#include "piece.h"
#include "scoreboard.h"
#include "textfont.h"
#include "time.h"

#define PIECESTARTX 2
#define PIECESTARTY 12
#define PIECESTARTZ 0

#define BOARD_WIDTH 6
#define BOARD_HEIGHT 14

#define FALLING_PIECE 0
#define GRAVITY_WORK 1
#define EXPLODING 2
#define CLEAR_BOARD 3
#define GAME_OVER_MAN 4

#define LIVES 3
#define PIECE_DONE_THRESH 0.1
#define GRAVITY_WAIT 0.5

class Display;

class Display
	{
	public:
	Display()
		{
		unsigned int now = time(NULL);
		srand( now ); 
		fallRate = PIECEMOVEY;
		for ( int i=0; i<BOARD_WIDTH; i++ )
			{
			for ( int j=0; j<BOARD_HEIGHT; j++ )
				{
				board[i][j] = -1;
				boardJewels[i][j] = NULL;
				explode[i][j] = 0;
				}
			}
		state = FALLING_PIECE;
		livesLeft = LIVES - 1;
		doneReDest = 0;
		timer = 0;
		explodeLevel = 0;
		gameLevel = 0;
		textFont = new TextFont();
		scoreBoard = new ScoreBoard( getTexture() );
		newPiece();
		}
	
	void clearBoard();
	void draw();
	void update();
	int updateBoard();
	void flip();
	void fastDown();
	void down();
	void left();
	void right();
	int getTopOfColumn( int colnum );
	void checkForMatches();

	GLubyte
	getTexture()
		{
		return textFont->getTexture();
		}

	void newPiece()
		{
		// printf( "making new piece\n" );
		explodeLevel = 0;
		piece = new Piece( PIECESTARTX, PIECESTARTY, PIECESTARTZ, fallRate );
		}
	
	Piece *getPiece()
		{
		return piece;
		}

	private:
	Piece *piece;
	float fallRate;
	int board[ BOARD_WIDTH ][ BOARD_HEIGHT ];
	Jewel *boardJewels[ BOARD_WIDTH ][ BOARD_HEIGHT ];
	int explode[ BOARD_WIDTH ][ BOARD_HEIGHT ];
	int explodeLevel;
	int gameLevel;
	int state;
	int livesLeft;
	int doneReDest;
	float timer;
	TextFont * textFont;
	ScoreBoard * scoreBoard;
	};
		
#endif	
