#ifndef JEWEL_H
#define JEWEL_H

#include "base.h"

#define THRESH 0.1

class Jewel;

class Jewel
	{
	public:
	Jewel( int newStyle, float origpos[3], float newFallRate )
		{
		style = newStyle;
		rate[0] = PIECEMOVEX;
		rate[1] = newFallRate;
		rate[2] = PIECEMOVEX;
		//printf( "style %d\n", style );
		
		shininess = 0.3;
		switch( style )
			{
			case 0:
				r_specularColor = 0.8;
				g_specularColor = 0.8;
				b_specularColor = 0.8;
				r_diffuseColor = 0.8;
				g_diffuseColor = 0.3;
				b_diffuseColor = 0.0;
				break;
			case 1:
				r_specularColor = 0.8;
				g_specularColor = 0.8;
				b_specularColor = 0.8;
				r_diffuseColor = 0.8;
				g_diffuseColor = 0.0;
				b_diffuseColor = 0.0;
				break;
			case 2:
				r_specularColor = 0.8;
				g_specularColor = 0.8;
				b_specularColor = 0.8;
				r_diffuseColor = 0.0;
				g_diffuseColor = 0.0;
				b_diffuseColor = 0.8;
				break;
			case 3:
				r_specularColor = 0.8;
				g_specularColor = 0.8;
				b_specularColor = 0.8;
				r_diffuseColor = 0.8;
				g_diffuseColor = 0.8;
				b_diffuseColor = 0.0;
				break;
			case 4:
				r_specularColor = 0.8;
				g_specularColor = 0.8;
				b_specularColor = 0.8;
				r_diffuseColor = 0.0;
				g_diffuseColor = 0.8;
				b_diffuseColor = 0.0;
				break;
			default:
				r_specularColor = 0.8;
				g_specularColor = 0.8;
				b_specularColor = 0.8;
				r_diffuseColor = 0.8;
				g_diffuseColor = 0.8;
				b_diffuseColor = 0.8;
				break;
			}	

		glfModelSpecular[0] = r_specularColor;
		glfModelSpecular[1] = g_specularColor;
		glfModelSpecular[2] = b_specularColor;

		for( int i=0; i<3; i++ )
			{
			pos[i] = origpos[i];
			dest[i] = (int) origpos[i];
			// printf( "[%d] original pos %f\n", i, pos[i] );
			}
		dest[1] = 0;
		}

	void draw()
		{
		glPushMatrix();
		switch( style )
			{
			case 0:
				draw0();
				break;
			case 1:
				draw1();
				break;
			case 2:
				draw2();
				break;
			case 3:
				draw3();
				break;
			case 4:
				draw4();
				break;
			default:
				draw0();
				break;
			}
		glPopMatrix();
		}

	void update();
	void updatePos( int modx, int mody, int modz );
	void draw0();
	void draw1();
	void draw2();
	void draw3();
	void draw4();
	void downFast();
	void quantitizePos();

	int getXDest()
		{
		return dest[0];
		}

	int getYDest()
		{
		return dest[1];
		}

	float getXPos()
		{
		return pos[0];
		}

	float getYPos()
		{
		return pos[1];
		}

	int getStyle()
		{
		return style;
		}

	void setYPos( int newypos )
		{
		pos[1] = (float) newypos;
		}

	void setYDest( int newydest )
		{
		dest[1] = newydest;
		}

	void updateXDest( int modxdest )
		{
		dest[0] += modxdest;
		}

	void setFallRate( float newFallRate )
		{
		rate[1] = newFallRate;
		}

	int isMoving()
		{
		int moving = 0;
		for( int i=0; i<3; i++ )
			{
			if ( pos[i] != (float) dest[i] )
				{
				moving = 1;
				}
			}
		return moving;
		}

	private:
	Jewel();

	int style;
	float r_specularColor;
	float g_specularColor;
	float b_specularColor;
			
	float r_diffuseColor;
	float g_diffuseColor;
	float b_diffuseColor;

	GLfloat glfModelSpecular[4];
	GLfloat shininess;

	float pos[3];
	int dest[3];
	float rate[3];

	};

#endif	
